
#include "inet.h"
#include "hsocket.h"

#define NLOOPS 1

/*
 * this test program demonstrates connecting to a remove avs module,
 * computing data then sending it over in an architecture-neutral format.
 *
 * the data computed in this example is contrived:
 *    z = t*(x**2 + y**2),x==-10..10,y==-10..10
 */

void
send_data_size_info(SocketFD *client_info,
		    int dims[5])
{
    void *config_xdr_buf;
    XDR config_xdr;
    int xdr_buf_size;		/* bytes */

/*    xdr_buf_size = xdr_sizeof(xdr_int,dims)*5; */
    
    /* NOTE: xdr_sizeof() not widely implemented.  */
    xdr_buf_size = SIZEOF_XDR_INT*5;

    config_xdr_buf = (void *)malloc(xdr_buf_size);

    xdrmem_create(&config_xdr,
		  (const caddr_t)(config_xdr_buf),
		  xdr_buf_size,
		  XDR_ENCODE);
    
    xdr_vector(&config_xdr,
	       (caddr_t)dims,
	       (const u_int)5,
	       (const u_int)sizeof(int),
	       xdr_int);

    client_send_message(client_info,
			config_xdr_buf,
			xdr_buf_size);

    free(config_xdr_buf);
    xdr_destroy(&config_xdr);

}

void
compute_data(double *data,
	     double t,
	     int isize,
	     int jsize)
{
    /* compute z = t*(x**2 + y**2) over -5 to 5 at isize,jsize resolution */
    int i,j;
    double u,du, v,dv;
    int indx=0;

    du = 10.0/(double)isize;
    v = -5.;
    dv = 10.0/(double)jsize;
    
    for (j=0;j<jsize;j++, v += dv)
    {
	u = -5.;
	for (i=0;i<isize;i++, u += du)
	    data[indx++] = t * (u*u + v*v);
    }
}

void
send_data(SocketFD *client_info,
	  double *data,
	  int dims[5])
{
    void *data_xdr_buf;
    XDR data_xdr;
    int data_buf_size;		/* bytes */
    int i;
    int ndata;

    for (ndata=1,i=0;i<5;i++)
	ndata *= (dims[i] == 0.) ? 1. : dims[i];

    /* assume that ndata != 0, or, in other words, that for each
     dims[i], that some dims[i] != 0, otherwise this code
     might blow up. */

/*    data_buf_size = xdr_sizeof(xdr_double,data)*ndata; */
    /* NOTE: xdr_sizeof() not widely implemented.  */
    
    data_buf_size = SIZEOF_XDR_DOUBLE*ndata;

    data_xdr_buf = (void *)malloc(data_buf_size);

    xdrmem_create(&data_xdr,
		  (const caddr_t)(data_xdr_buf),
		  data_buf_size,
		  XDR_ENCODE);
    
    xdr_vector(&data_xdr,
	       (caddr_t)data,
	       (const u_int)ndata,
	       (const u_int)sizeof(double),
	       xdr_double);

    client_send_message(client_info,
			data_xdr_buf,
			data_buf_size);

    free(data_xdr_buf);
    xdr_destroy(&data_xdr);

}

main(int argc,
     char *argv[])
{
    SocketFD client_info;
    int status,i;
    clock_t ck;
    char *time_buf;
    int dims[5];
    double *data,t;

    if (argc == 1)
    {
      status = open_client_socket(&client_info,SERV_HOST_NAME,SERV_TCP_PORT);
    }
    else /* assume host and port are specified */
    {
      printf("Server = %s, port = %d\n",argv[1],atoi(argv[2]));
      status = open_client_socket(&client_info,argv[1],atoi(argv[2]));
    }

    if (status == 1)
	fprintf(stderr,"  connected. \n");

    dims[0] = dims[1] = 20;
    dims[2] = dims[3] = dims[4] = 0;
    data = (double *)malloc(sizeof(double)*20*20);

    for (i=0,t=0. ; i < 2; i++,t+= 1.0)
    {
	
	/* send data config information */
	send_data_size_info(&client_info,dims);

	compute_data(data,t,20,20);

	send_data(&client_info,data,dims);


    }

    status = close_client_socket(&client_info);
}
