
/*
 * Copyright (C) 1998,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 

#include "inet.h"
#include "hsocket.h"


int
receive_data(int sockfd)
{
#if 0
    XDR config_xdr, data_xdr;
    void *config_xdr_buf, *data_xdr_buf;
    int dims[5];
    int size;
    char junk;

     size = SIZEOF_XDR_INT * 5; 
#if VERBOSE
    fprintf(stderr," size of config_xdr_buf is %d. \n",size);
#endif

    config_xdr_buf = (void *)malloc(size);
    
    read_sock(sockfd, config_xdr_buf, size);
    write_sock(sockfd,&junk,1);

    xdrmem_create(&config_xdr,
		  (const caddr_t)(config_xdr_buf), /* holding buffer */
		  SIZEOF_XDR_INT*5, /* ?? */
		  XDR_DECODE);
	
    xdr_vector(&config_xdr,
	       (caddr_t)dims,
	       (const u_int)5,
	       (const u_int)sizeof(int),
	       xdr_int);

#if VERBOSE
    fprintf(stderr," size of array is %d %d %d %d %d \n",dims[0],dims[1],dims[2],dims[3],dims[4]);
#endif

    free(config_xdr_buf);

#endif
}

void
to_data_size(char *msg,
	     int dims[5])
{
    void *config_xdr_buf;
    XDR config_xdr;
    int xdr_buf_size;		/* bytes */


    config_xdr_buf = msg;

    xdrmem_create(&config_xdr,
		  (const caddr_t)(config_xdr_buf),
		  SIZEOF_XDR_INT*5, /* a priori knowledge */
		  XDR_DECODE);
    
    xdr_vector(&config_xdr,
	       (caddr_t)dims,
	       (const u_int)5,
	       (const u_int)sizeof(int),
	       xdr_int);

    free(config_xdr_buf);
    xdr_destroy(&config_xdr);

}

void
get_data(char *msg,
	 double *data,
	 int ndata)
{
    void *data_xdr_buf;
    XDR data_xdr;
    int data_buf_size;		/* bytes */


    data_xdr_buf = msg;

    memset(&data_xdr,0,sizeof(XDR));

    xdrmem_create(&data_xdr,
		  (const caddr_t)(data_xdr_buf),
		  SIZEOF_XDR_DOUBLE*ndata, /* a priori knowledge */
		  XDR_DECODE);

    if (data_xdr.x_ops == NULL)
	fprintf(stderr," failed to malloc memory in get_data call. \n");
    
    xdr_vector(&data_xdr,
	       (caddr_t)data,
	       (const u_int)ndata,
	       (const u_int)sizeof(double),
	       xdr_double);

    free(data_xdr_buf);
    xdr_destroy(&data_xdr);
}


main(int ac,
     char *av[])
{
    SocketFD server_info, client_info;
    
    int status;
    char *msg;
    int msg_length;
    int command_opcode;
    char command[8];
    int dims[5];
    int ndata,i;
    double *data;
    timeout_thingy timeout = {1000,0};

    status = open_server_socket(&server_info,SERV_TCP_PORT);
    fprintf(stderr," waiting for connection...");
    status = server_waitfor_connection(&server_info,&client_info,NULL);
    
    for (;status==1;)
    {

	fprintf(stderr, "\n\n ***********  new data ************ \n\n");

	
	server_receive_message(&client_info,command,&msg,&msg_length,&timeout);
	if (strcmp(command,"clos") == 0)
	    break;
	
	to_data_size(msg,dims);	


	fprintf(stderr,"size of array is %d %d %d %d %d \n",dims[0],dims[1],dims[2],dims[3],dims[4]);

	for (i=0,ndata=1;i<5;i++)
	    ndata *= (dims[i] == 0.) ? 1 : dims[i];

	data = (double *)malloc(sizeof(double)*ndata);

	server_receive_message(&client_info,command,&msg,&msg_length,&timeout);
	if (strcmp(command,"clos") == 0)
	    break;

	get_data(msg,data,ndata);

	fprintf(stderr," received data ok. \n");
	

	status = 1;
	free((void *)data);

    }
#if VERBOSE
    fprintf(stderr," now closing it.\n");
#endif
    server_close_connection(&client_info);

}
