

/*
 * Copyright (C) 1998,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 

#include <stdio.h>
#define _HSOCKET_SRC

#include "hsocket.h"

#define BSIZE 4096

int sig_flag = 0;
static int one=1;

static timeout_thingy global_timeout={5000,0};

void
sigio_func(int foo)
{
    sig_flag = 1;
}

int
select_on_read(SocketFD *c,
	       timeout_thingy *timeout)
{
    int status;
    fd_set read_fds, write_fds, except_fds;
    struct timeval wait;

    /*
     * this routine will block until a "ready to read" signal is
     * received from the socket.  a maximum of 5 minutes will be
     * spent waiting.  if this timeout is reached, this routine will
     * return a 0 to indicate an error.  otherwise, it will return a 1.
     */
    if (timeout == NULL)
    {
	wait.tv_sec = 60*5;		/* five minutes */
	wait.tv_usec = 0;
    }
    else
    {
	wait.tv_sec = timeout->seconds;
	wait.tv_usec = timeout->microseconds;
    }
    FD_ZERO(&read_fds);

    FD_SET(c->socket_id,&read_fds);

    /*
     * do the select - waiting for an event on the socket.
     */
    status = select(c->socket_id + 1, &read_fds, NULL, NULL,   &wait);

    /*
     * if status == 1, there's an event. if status == 0, there was
     * a timeout.
     */

    if (status == 0)
	return(-1);
    else
	return(1);

}

int
open_server_socket(SocketFD *s,
		   int portnum)
{
    /*
     * creates a socket, binds it, goes passive with a listen,
     * then returns control to the caller.
     */
    int sockfd;
    struct sockaddr_in serv_addr;

    signal(SIGIO,sigio_func);
    signal(SIGPIPE,SIG_IGN);

    if ((sockfd = socket(AF_INET,SOCK_STREAM,0)) < 0)
    {
	perror("socket:");
	return(0);
    }

    setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, (char *)&one,sizeof(int));
    s->socket_id = sockfd;

    memset(&serv_addr,0,sizeof(serv_addr));
    serv_addr.sin_family = AF_INET;
    serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
/*    serv_addr.sin_port = htons(SERV_TCP_PORT); */
    serv_addr.sin_port = htons(portnum);
    
    if (bind(sockfd, (struct sockaddr *)&serv_addr,
	     sizeof(serv_addr)) < 0)
    {
	perror("bind:");
	return(0);
    }

    memcpy(&(s->socket_addr),&serv_addr,sizeof(struct sockaddr_in));

    return(1);			/* ok */
}

int
server_waitfor_connection(SocketFD *s,
			  SocketFD *c,
			  timeout_thingy *timeout)
{
    int clilen,newsockfd;
    int status;
    struct sockaddr_in cli_addr;
    
    clilen = sizeof(cli_addr);

    status = 1;
    if (timeout != NULL)
	status = select_on_read(s,timeout);

    if (status <= 0)
	return(status);		/* error or timeout. */
    
    listen(s->socket_id,5);
    newsockfd = accept (s->socket_id, (struct sockaddr *) &cli_addr,
			    &clilen);

    if (newsockfd < 0)
    {
	perror("accept");
	return(0);
    }

    c->socket_id = newsockfd;
    memcpy(&(c->socket_addr),&cli_addr,clilen);

    return(1);
}

int
server_close_connection(SocketFD *c)
{
    close(c->socket_id);
    return(1);
}

int
open_client_socket(SocketFD *c,
		   char *hostname,
		   int portnum)
{
    int sockfd;
    struct sockaddr_in serv_addr;
    struct hostent *h;

    signal(SIGIO,sigio_func);
    signal(SIGPIPE,SIG_IGN);

    memset(&serv_addr,0,sizeof(struct sockaddr_in));

/*    h = gethostbyname(SERV_HOST_NAME); */
    h = gethostbyname(hostname);

    /* some error checking would be nice */

    if (h == NULL)
    {
#ifdef KOPSYS_IRIX
	herror("opening client socket:");
#else
	perror("gethostbyname:");
#endif
	return(0);
    }
    
    
    serv_addr.sin_family = AF_INET;
/*    serv_addr.sin_addr.s_addr = inet_addr(SERV_HOST_ADDR); */
    memcpy(&((serv_addr.sin_addr)),h->h_addr,4);
/*    memcpy(&((serv_addr.sin_addr.s_addr)),h->h_addr,4); */
/*    serv_addr.sin_addr.s_addr = h->h_addr;*/
    
/*    serv_addr.sin_port = htons(SERV_TCP_PORT);*/
    serv_addr.sin_port = htons(portnum);

    if ((sockfd = socket(AF_INET,SOCK_STREAM,0)) < 0)
    {
	perror("socket:");
	return(0);
    }

    setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, (char *)&one,sizeof(int));

    if (connect(sockfd,(struct sockaddr *)&serv_addr,
		sizeof(serv_addr)) < 0)
    {
	perror("connect:");
	return(0);
    }

    c->socket_id = sockfd;
    memcpy(&(c->socket_addr),&serv_addr,sizeof(serv_addr));
    return(1);
}


int
close_client_socket(SocketFD *c)
{
    /*
     *  tell the other side that we're going to shut down.
     */
    command_atom p;
    int status;
    int junk;
    
    memcpy(&p,atoms+CLOSE,sizeof(command_atom));

    status = my_send(c, (char *)&p, sizeof(command_atom));

    status = my_recv(c,&junk,1,&global_timeout);

    /* wait for an ack */
    
    close(c->socket_id);
    return(1);
}

static int
macho_recv(int fd,
	   char *buf,
	   int count)
{
    int in_the_bag = 0;
    int bytes_remaining;
    int rstat = 0;

    bytes_remaining = count;
    for (;bytes_remaining > 0;)
    {
	rstat = recv(fd,buf+in_the_bag,bytes_remaining,0);
	bytes_remaining -= rstat;
	in_the_bag += rstat;
	if (rstat == 0)
	    break;
    }
    return(in_the_bag);
}

static int
my_recv(SocketFD *s,
	char *buf,
	int nbytes,
	timeout_thingy *timeout)
{
    /*
     * returns > 0 for success,
     * 0 for failure,
     * -1 for timeout
     */
    int i,index=0;
    int whole_bufs, partial_bufs;
    int whole_bufsize, partial_bufsize;
    int status;

    whole_bufs = nbytes / BSIZE;
    whole_bufsize = BSIZE;
    
    partial_bufs = ((nbytes % BSIZE) == 0 ? 0 : 1);
    if (partial_bufs)
	partial_bufsize = nbytes % BSIZE;

#if 0
    if (select_on_read(s,to) == 0)
	return(0);
#endif
    i = select_on_read(s,timeout);

    if (i <= 0)
	return(i);
	
    for (i=0;i<whole_bufs;i++, index += whole_bufsize)
    {
	status = macho_recv(s->socket_id, buf+index, whole_bufsize);
	if (status != whole_bufsize) {
	    fprintf(stderr," error in recv \n");
	    fprintf(stderr,"status=%d, bufsize=%d\n",status,whole_bufsize);
	}
	
    }

    for (i=0;i<partial_bufs; i++)
    {
	status = macho_recv(s->socket_id, buf+index, partial_bufsize);
	if (status != partial_bufsize) {
	    fprintf(stderr," error in recv: partial receive \n");
	    fprintf(stderr,"status=%d, bufsize=%d\n",status,partial_bufsize);
	}
    }

#if VERBOSE
    fprintf(stderr,"nbytes before return: %d\n",nbytes);
#endif
    return(nbytes);
}

static int
my_send(SocketFD *s,
	char *buf,
	int nbytes)
{
    int status;
    int i,index=0;
    int whole_bufs, partial_bufs;
    int whole_bufsize, partial_bufsize;

    whole_bufs = nbytes / BSIZE;
    whole_bufsize = BSIZE;
    
    partial_bufs = ((nbytes % BSIZE) == 0 ? 0 : 1);
    if (partial_bufs)
	partial_bufsize = nbytes % BSIZE;

    for (i=0;i<whole_bufs;i++, index += whole_bufsize)
    {
	status = send(s->socket_id, buf+index, whole_bufsize, 0);
	
	if (status < 0) {
	    fprintf(stderr, " error in send. \n");
	}
	    fprintf(stderr,"full send: status=%d, bufsize=%d\n",status,whole_bufsize);
    }

    for (i=0;i<partial_bufs; i++)
    {
	status = send(s->socket_id, buf+index, partial_bufsize, 0);
	if (status < 0) {
	    fprintf(stderr, " error in send. \n");
	}

#if VERBOSE
	    fprintf(stderr,"partial send: status=%d, bufsize=%d\n",status,partial_bufsize);
#endif
    }
    return(1);
}


int
client_send_message(SocketFD *c,
		    char *msg,
		    int msg_length)
{
    /*
     * sending a bulk message is a two-stage process.  first, we tell
     * the server how many bytes we're going to send over.  we get
     * an ack, then we send over a pile of data.
     */
    int nbytes;
    int status;
    command_atom p;
    u_long tmp;
    short tshort;

    /*
     * first, tell the other side we want to xfer a bunch of data,
     * and tell it how stuff is coming over.
     */

    /* stuff the command string in. */
    memcpy(&p,atoms+XFER,sizeof(command_atom));

    /* stuff the length of the message (in bytes) */
    p.msglength[0] = (unsigned char)(msg_length & 0xff);
    p.msglength[1] = (unsigned char)((msg_length >> 8) & 0xff);
    p.msglength[2] = (unsigned char)((msg_length >> 16) & 0xff);
    
/*    tmp = msg_length; */
#if VERBOSE    
    fprintf(stderr," outgoing message length is %d \n",msg_length);
#endif

    status = my_send(c,(char *)&p, ATOM_SIZE);
    
#if VERBOSE
    fprintf(stderr,"client sends size info...");
#endif
    
    if (status < 0)
	perror("send");
    
    /* put check in here comparing bytes written with expected
     number of bytes to write. */

    /* do a listen for the ack. */
    status = my_recv(c,(char *)&p,ATOM_SIZE,&global_timeout);
	
#if VERBOSE
    if (strncmp(p.command,atoms[ACK].command,strlen(atoms[ACK].command)) != 0)
	fprintf(stderr," expected an ACK.  got a <%s> \n",p.command);
    fprintf(stderr,"ack received. \n");
    fflush(stderr);
#endif

    /* check to see that it was an ack. */

    /* if it was an ack, go ahead and send over the raw data. */
/*     status = send(c->socket_id, msg, msg_length, 0); */
    status = my_send(c,msg,msg_length);

#if VERBOSE
    fprintf(stderr,"client sends data..");
    fflush(stderr);
#endif


    status = my_recv(c, (char *)&p, ATOM_SIZE,&global_timeout);

#if VERBOSE
	fprintf(stderr,"ack received. \n");
	fflush(stderr);
#endif

    
    return(1);
    
}

static int
parse_command(char *c)
{
    int icommand;
    
    if (strcmp(c,atoms[CLOSE].command) == 0) 
	icommand = CLOSE;
    else if (strcmp(c,atoms[XFER].command) == 0)
	icommand = XFER;
    else if (strcmp(c,atoms[ACK].command) == 0)
	icommand = ACK;
    return(icommand);
}
	

int
server_receive_message(SocketFD *c,
		       char *command,
		       char **msg,
		       int *msg_length,
		       timeout_thingy *timeout)
{
    int nbytes;
    int status;
    command_atom p,pout;
    int icommand;

    /*
     * returns 1 for success,
     * 0 for error,
     * -1 for timeout
     */

    status = my_recv(c, (char *)&p, ATOM_SIZE, timeout);

    if (status <= 0)		/* timeout or error */
	return(status);

#if VERBOSE
    fprintf(stderr," command is %s \n",p.command);
#endif

    icommand = parse_command(p.command);
    strcpy(command,p.command);

    /* put check in here comparing bytes written with expected
     number of bytes to write. */

    /*
     * return an ACK to the sender. 
     */
    memcpy(&pout,atoms+ACK,sizeof(command_atom));

    status = my_send(c,(char *)&pout, ATOM_SIZE);

    if (status < 0)
	perror("send");

    switch(icommand)
    {
    case CLOSE:
	break;
    case XFER:
	{
	    u_long tmp;
	    int i;
	    char *buf;


	    i = (int)(p.msglength[0]);
	    i |= (int)(p.msglength[1] << 8);
	    i |= (int)(p.msglength[2] << 16);

#if VERBOSE
	    fprintf(stderr,"\tserver: incoming message length will be %d bytes. \n",i);
#endif
	    
	    buf = (char *)malloc(sizeof(char)*i);
	    
	    status = my_recv(c, buf, i,&global_timeout);

	    /* send an ack. want to send # bytes read? */
/*	    status = send(c->socket_id, (char *)&pout, 8,0); */

	    status = my_send (c,(char *)&pout, ATOM_SIZE);

	    *msg = buf;
	    *msg_length = i;
	    
	}
	break;
    case ACK:
	break;
    }
    
    return(1);
    
}

