
/*
 * Copyright (C) 1997,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 


#ifndef _hsocket_h_
#define _hsocket_h_


	/*-------------------------------------*
	|       #include
	---------------------------------------*/

#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <signal.h>
#include <sys/uio.h>

	/*-------------------------------------*
	|       #defines
	---------------------------------------*/

/* used during testing only */
/* #define SERV_HOST_ADDR "131.243.240.67" */

/* CHANGE these two for your machine. these are the only user-configurable
 parameters in this file. */
#define SERV_TCP_PORT 8118
#define SERV_HOST_NAME "bozo.lbl.gov" 

/*
 * misc commands
 */

#define CLOSE     0		/* closes the connection */
#define XFER      1		/* xfer a huge amount of data */
#define ACK       2


	/*-------------------------------------*
	|       typedefs
	---------------------------------------*/

typedef struct command_atom
{
    char command[5];		/* the actual command */

    unsigned char msglength[3]; /* max msg length = 2**24-1 bytes (16Mbytes) */

    char pad[8];
    
} command_atom;

#define ATOM_SIZE sizeof(command_atom)

typedef struct timeout_thingy
{
    int seconds;
    int microseconds;
} timeout_thingy;

typedef struct SocketFD
{
    int socket_id;
    struct sockaddr_in socket_addr;
} SocketFD;

	/*-------------------------------------*
	|       global variable declarations
	---------------------------------------*/

static command_atom atoms[4]=
{
    { {"clos"},{"123"},{"12345678"} },
    { {"data"},{"123"},{"12345678"} },
    { {"ackn"},{"123"},{"12345678"} }
};


	/*-------------------------------------*
	|       macros
	---------------------------------------*/


	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

int open_server_socket(SocketFD *s, int port);
int server_waitfor_connection(SocketFD *s,SocketFD *c, timeout_thingy *);
int server_close_connection(SocketFD *c);

int open_client_socket(SocketFD *s, char *hostname, int portnum);
int close_client_socket(SocketFD *c);

int server_receive_message(SocketFD *,char *,char **,int *, timeout_thingy *);
int client_send_message(SocketFD *c,char *msg,int msg_length);

int set_socket_buffersize(SocketFD *s);


#endif /* _hsocket_h_ */
/* Don't add after this point */
